'25/6/2013 20:03
'------------------------------------------------------------------------------'
'Peak Power Tracking Solar Charger for Maximite Computer
'Based on Arduino Peak Power Tracking Solar Charger by Tim Nolan (www.timnolan.com)   5/1/09
'
'-------------------------------------------------------------------------------
'CONSTANTS:

SOL_AMPS_CHAN=1         'adc input for solar amps
SOL_V_CHAN=3            'adc input for solar volts
BAT_V_CHAN=2            'adc input for battery volts
AVG_NUM=20              'number of adc readings to average
PWM_PIN=9               'the output pin for the pwm
PWM_ENABLE_PIN=11       'used to control shutoff of the IR2184 MOSFET driver
PWM_MAX=99              'the value for pwm duty cyle 0-100%
PWM_MIN=40              'the value for pwm duty cyle 0-100%
PWM_START=60            'the value for pwm duty cyle 0-100%
DELTA=0.3               'the value the increment to the pwm value for the ppt algorithm
LOW_SOL_WATTS=5
MIN_SOL_WATTS=1
MIN_BAT_V = 22.0
MAX_BAT_V = 28.2
HIGH_BAT_V= 26.0
OFF_NUM=50               ' number of iterations of off charger state
FREQ = 20000
ledPin5 = 5           ' LED
'---------------------------------
'VARIABLES:
count = 0
pwm = 0                  'pwm duty cycle 0-100%
old_sol_watts = 0        'solar watts from previous time through ppt routine scaled by 100
led_counter = 0

'**************************
'* START EXECUTION HERE:  *
'**************************
setup

do while 1=1 '@
looop
loop

'---------------------------------------------------------------------------
' This routine is automatically called at powerup/reset
'---------------------------------------------------------------------------
Sub setup                      ' run once, when the charger starts
  Cls
  SetPin ledPin5, 8            ' sets the digital pin as output
  SetPin PWM_ENABLE_PIN, 8     ' sets the digital pin as output
  SetPin SOL_AMPS_CHAN, 1      ' sets the ANALOG pin as INput
  SetPin SOL_V_CHAN, 1         ' sets the ANALOG pin as INput
  SetPin BAT_V_CHAN, 1         ' sets the ANALOG pin as INput
  Pin(PWM_ENABLE_PIN)=1        ' turn on MOSFET driver chip
  PWM FREQ,PWM_START           ' starting value for pwm
  charger_state$ = "on"        ' start with charger state as on
End Sub

'------------------------------------------------------------------------------
'This routine reads and averages the analog inputs for this system, solar volts, solar amps
'and battery volts. It is called with the adc channel number (pin number) and returns the
'average adc value as an integer.
'-------------------------------------------------------------------------------
Function read_adc(channel)

  Local sum,temp,i
  sum = 0
  For i=0 To AVG_NUM              ' read raw adc values AVG_NUM number of times

    sum = sum + Pin(channel)      ' store sum for averaging
'   For temp=1 To 100: Next temp  ' pauses for 50 microseconds
  Next i
  read_adc = sum / AVG_NUM    'divide sum by AVG_NUM To get average And return it
End Function

'------------------------------------------------------------------------------
'This routine sets the pwm duty cycle.
'There is a special case for 100% duty cycle.
 Normally this would be have the top MOSFET on all the time but the
 MOSFET driver IR2104 uses a charge pump To generate the gate
'voltage so it has to keep running all the time. So for 100% duty cycle
 I set the pwm value To PWM_FULL so it is on 99% almost full on but
 is switches enough To keep the charge pump on IR2104 working.
'------------------------------------------------------------------------------
Sub set_pwm_duty
  If pwm > PWM_MAX Then
     pwm = PWM_MAX           'check limits of PWM duty cyle and set to PWM_MAX
  ElseIf pwm < PWM_MIN Then
     pwm=PWM_MIN             'if pwm is less than PWM_MIN then set it to PWM_MIN
  EndIf
  If (pwm < PWM_MAX) Then
     PWM FREQ,pwm
  ElseIf pwm = PWM_MAX Then
     PWM (FREQ/2),PWM_MAX    'set pwm duty cycle at 50uS period
'must keep switching so set duty cycle at 99% and slow to half rate
  EndIf
End Sub

'-----------------------------------------------------------------------
' This routine prints the data.
'-----------------------------------------------------------------------
Sub print_data
  Locate 0,50
  Print "FREQ = ";FREQ;
  Print "  charger = ";charger_state$;"  "
Print
  Print "       pwm = ";Format$(pwm,"%6.1f")
  Print "  sol_amps = ";Format$(sol_amps,"%6.2f")
  Print " sol_watts = ";Format$(sol_watts,"%6.2f")
  Print " SOL_V = ";Format$(SOL_V,"%6.2f")
  Print " BAT_V = ";Format$(BAT_V,"%6.2f")
Print "off_count= "; off_count ;" "
If (BAT_V > MIN_BAT_V) And (BAT_V < MAX_BAT_V) And (SOL_V > BAT_V) Then
  Print "all ok"
  Else
    Print (SOL_V > BAT_V)
  EndIf
x=x+1
If x>400 Then
 x=0
 Cls
 EndIf
Pixel(100+3*pwm,400-sol_watts*2.5)=1
End Sub

'------------------------------------------------------------------------------
' This routine reads all the analog input values.
' It also calculates the input watts from solar amps * solar voltage
' actual volts=v.read * (r1+r2)/r2.
' for 24v batt we need max reading of 32v. Assuming 3.3v reference,
' factor needed = 9.697. I use 22k and 2k2 for factor of 11 giving full scale
' of 36.3v . We need max of 50v on solar side so factor needed=15.15 .
' I use 33k and 2k2 gives ratio of 16.00
'------------------------------------------------------------------------------
Sub read_data
  sol_amps =  read_adc(SOL_AMPS_CHAN)  * 7.5
    '50amp max..input
  SOL_V = read_adc(SOL_V_CHAN) * 16 * 0.95
  BAT_V = read_adc(BAT_V_CHAN) * 11 * 0.95
  sol_watts = sol_amps * SOL_V
End Sub

'------------------------------------------------------------------------------
' This routine blinks the LED.
'------------------------------------------------------------------------------
Sub blink_leds

    If Pin(led_Pin_jc2)=1  Then
      Pin(ledPin5)=0   ' set LED off
    Else
      Pin(ledPin5)=1   ' set LED on
    EndIf
End Sub

'------------------------------------------------------------------------------
' This routine is the charger state machine. It has four states on, off, bulk and float.
' It's called once each time through the main loop to see what state the charger should be in.
'
'On State - this is charger state for MIN_SOL_WATTS < solar watts < LOW_SOL_WATTS. This state is
'probably happening at dawn and dusk when the solar watts input is too low for the bulk charging
'state but not low enough to go into the off state. In this state we just set the pwm = 100% to
'get the most of low amount of power available.
'
'Bulk State - this is charger state for solar watts > MIN_SOL_WATTS. This is where we do the bulk of
'the battery charging and where we run the Peak Power Tracking alogorithm. In this state we try and
'run the maximum amount of current that the solar panels are generating into the battery.

'Float State - As the battery charges it's voltage rises. When it gets to the MAX_BAT_V we are
'done with the bulk battery charging and enter the battery float state. In this state we try to
'keep the battery voltage at MAX_BAT_V by adjusting the pwm value. If we get to pwm = 100%
'it means we can't keep the battery voltage at MAX_BAT_V which probably means the battery is
'being drawn down by some load so we need to back into the bulk charging mode.

'Off State - This is state that the charger enters when solar watts < MIN_SOL_WATTS. The charger
'goes into this state when it gets dark and there is no more power being generated by the solar
'panels. The MOSFETs are turned off in this state so that power from the battery doesn't leak back
'into the solar panel. When the charger off state is first entered all it does is decrement
'off_count for OFF_NUM times. This is done because if the battery is disconnected (or battery fuse
'is blown) it takes some time before the battery voltage changes enough so we can tell that
'the battery is no longer connected. This off_count gives some time for battery voltage to
'change so we can tell this.
'------------------------------------------------------------------------------
Sub run_charger

   'off_count = OFF_NUM

'----------
If charger_state$="on" Then
  If sol_watts < MIN_SOL_WATTS Then    'if watts input from the solar panel is less than
     charger_state$ = "off"            'the minimum solar watts then it is gettig dark so
     off_count = OFF_NUM               'go to the charger off state
     Pin(PWM_ENABLE_PIN)=0
  ElseIf BAT_V > MAX_BAT_V Then        'else if the battery voltage has gotten above the float
     charger_state$ = "float"          'battery float voltage go to the charger battery float state
  ElseIf sol_watts < LOW_SOL_WATTS Then  'else if the input watts is less than low solar watts
     pwm = PWM_MAX                     'it means there is not much power being generated
     set_pwm_duty                      'set the pwm = 100% so we get as much power as possible
                                       'and stay in the charger on state
      Else
        pwm = BAT_V * 100 / SOL_V + 5     'else if we are making more than low solar watts figure
        set_pwm_duty                      'out what the pwm value should be
        charger_state$ = "bulk"           'and change the charger state
      EndIf
'---------
 ElseIf charger_state$="bulk" Then
       If sol_watts < MIN_SOL_WATTS Then  'if watts input is less than
          charger_state$ = "off"          'the minimum solar watts then it is getting dark so
          off_count = OFF_NUM             'go to the charger off state
          Pin(PWM_ENABLE_PIN)=0
       ElseIf BAT_V > MAX_BAT_V Then      'else if the battery voltage is now above the
          charger_state$ = "float"        'battery float voltage change to float state
          DELTA= Abs(DELTA)
       ElseIf sol_watts < LOW_SOL_WATTS Then  'else if the solar input watts is less than
          charger_state$ = "on"               'low solar watts it means there is not much
                                              'power being generated by the solar panel
          Pin(PWM_ENABLE_PIN)=1               'so go to charger on state
'this is where we do the Peak Power Tracking or Maximum Power algorithm
       Else
         If (old_sol_watts >= sol_watts) Then
           DELTA = -DELTA        ' DELTA to make pwm increase or decrease to maximize watts
        EndIf
         pwm = pwm + DELTA          ' add DELTA to change PWM duty cycle for PPT algorythm
         old_sol_watts = sol_watts  ' load old_watts with current watts value for next time
         set_pwm_duty               ' set pwm duty cycle to pwm value
      EndIf
'---------
 ElseIf charger_state$="float" Then
       If sol_watts < MIN_SOL_WATTS Then  'if watts input from the solar panel is less than
          charger_state$ = "off"          'the minimum solar watts then it is getting dark so
          off_count = OFF_NUM             'go to the charger off state
          set_pwm_duty
          Pin(PWM_ENABLE_PIN)=0
       ElseIf BAT_V > MAX_BAT_V Then  'since we're in the float state if the batt voltage
          PWM = PWM - DELTA           'is above the float voltage back off the pwm
          set_pwm_duty
       ElseIf BAT_V < MAX_BAT_V Then  'else if the batt voltage is less than the float voltage
          PWM = pwm + DELTA           'increase pwm to get it back up to the float voltage
          set_pwm_duty
          If (pwm >= PWM_MAX) Then    'if pwm gets up to 100 it means we can't keep the batt at
             charger_state$ = "bulk"  'float voltage so jump to bulk state to charge the battery
          EndIf
       EndIf
 ElseIf charger_state$="off" Then
       'when we jump into the charger off state, off_count is set with OFF_NUM
  If off_count > 0 Then
      off_count = off_count - 1
' this is to allow the battery voltage to settle down to see if the
' battery has been disconnected
  ElseIf (BAT_V>HIGH_BAT_V) And (BAT_V<MAX_BAT_V) And (SOL_V>BAT_V) Then
        charger_state$ = "float"       'if batt voltage is still high and solar volts are high
        set_pwm_duty                   'change charger state to battery float
        Pin(PWM_ENABLE_PIN)=1
  ElseIf (BAT_V>MIN_BAT_V) And (BAT_V<MAX_BAT_V) And (SOL_V>BAT_V) Then
        pwm = BAT_V * 100 / SOL_V + 5
'if battery volts aren't quite so high but we have solar volts
        set_pwm_duty                   'greater than battery volts showing it is day light then
        charger_state$ = "on"          'change charger state to on so we start charging
        Pin(PWM_ENABLE_PIN)=1
        Pause(500)
  EndIf                                'else stay in the off state

'if charger state is not one of the above, default:
 Else
      Pin(PWM_ENABLE_PIN)=0
EndIf
End Sub

'----------------------------------------------------------------
' Main loop.
' Right now the number of times per second that this main loop runs
' is set by how long the pause statement takes. You can also run the
' commented out code and the charger routines will run once a second.
'----------------------------------------------------------------
Sub looop                          ' run over and over again

'  blink_leds                        'blink the heartbeat led
Pause 30
  read_data                         'read data from inputs
  run_charger                      'run the charger state machine
  print_data
  If Inkey$<>"" Then
    Pin(pwm_enable_pin)=0
    End
  EndIf

End Sub
'--------------------------------------------------------------------------
